/*
 * Copyright (c) 2019 Couchbase, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.couchbase.client.java.kv;

import com.couchbase.client.core.api.kv.CoreSubdocMutateCommand;
import com.couchbase.client.core.msg.kv.SubdocCommandType;
import com.couchbase.client.java.codec.JsonSerializer;

/**
 * An intention to perform a SubDocument replace operation.
 *
 * @author Graham Pople
 * @since 1.0.0
 */
public class Replace extends MutateInSpec {
    private final String path;
    private final Object doc;
    private boolean xattr = false;
    private boolean expandMacro = false;

    Replace(String path, Object doc) {
        this.path = path;
        if (doc instanceof MutateInMacro) {
            this.doc = ((MutateInMacro) doc).value();
            expandMacro = true;
            xattr = true;
        }
        else {
            this.doc = doc;
        }
    }

    /**
     * Sets that this is an extended attribute (xattr) field.
     * @return this, for chaining
     */
    public Replace xattr() {
        xattr = true;
        return this;
    }

    @Override
    public CoreSubdocMutateCommand toCore(JsonSerializer serializer) {
        SubdocCommandType command = path.equals("") ? SubdocCommandType.SET_DOC : SubdocCommandType.REPLACE;
        return new CoreSubdocMutateCommand(
            command,
            path,
            serializer.serialize(doc),
            false,
            xattr,
            expandMacro
        );
    }
}
